@@ -38,18 +38,18 @@
 # classes to allow runtime reconfiguration. Documentation of a node's
 # parameters is a handy byproduct.
 
-## @todo
+# @todo
 # Need to check types of min max and default
 # Need to put sane error on exceptions
 
 from __future__ import print_function
 
-from string import Template
-import os
 import inspect
-import string
-import sys
+import os
 import re
+from string import Template
+import sys
+import textwrap
 
 # LINEDEBUG="#line"
 LINEDEBUG = "//#line"
@@ -76,7 +76,7 @@ def check_name(name):
         raise Exception("The name of field \'%s\' does not follow the ROS naming conventions, see http://wiki.ros.org/ROS/Patterns/Conventions" % name)
 
 
-class ParameterGenerator:
+class ParameterGenerator(object):
     minval = {
         'int': -0x80000000,  # 'INT_MIN',
         'double': '-std::numeric_limits<double>::infinity()',
@@ -98,7 +98,7 @@ class ParameterGenerator:
         'bool': False,
     }
 
-    class Group:
+    class Group(object):
         instances = {}
 
         def __init__(self, gen, name, type, state, id, parent):
@@ -139,8 +139,9 @@ class ParameterGenerator:
                 'srcfile': inspect.getsourcefile(inspect.currentframe().f_back.f_code),
                 'edit_method': edit_method,
             }
-            if type == str_t and (max is not None or min is not None):
-                raise Exception("Max or min specified for %s, which is of string type" % name)
+            if (paramtype == str_t or paramtype == bool_t) and (max is not None or min is not None):
+                raise Exception(
+                    "Max or min specified for %s, which is of '%s' type" % (name, paramtype))
             check_name(name)
             self.gen.fill_type(newparam)
             self.gen.check_type_fill_default(newparam, 'default', self.gen.defval[paramtype])
@@ -246,21 +247,19 @@ class ParameterGenerator:
         id = 1
         self.constants = []
         if len(sys.argv) < 5:
-            msg = """
-ahhhh! Unexpected command line syntax!
+            raise SystemExit(textwrap.dedent("""\
+            ahhhh! Unexpected command line syntax!
 
-Are you trying to call a dynamic_reconfigure configuration generation script
-directly? When you are using dynamic_reconfigure with python, you don't ever
-need to invoke the configuration generator script yourself; it loads
-automatically. If you are using dynamic_reconfigure from C++, you need to
-add a call to generate_dynamic_reconfigure_options() in your CMakeLists.txt
+            Are you trying to call a dynamic_reconfigure configuration generation script
+            directly? When you are using dynamic_reconfigure with python, you don't ever
+            need to invoke the configuration generator script yourself; it loads
+            automatically. If you are using dynamic_reconfigure from C++, you need to
+            add a call to generate_dynamic_reconfigure_options() in your CMakeLists.txt
 
-For an example, see http://wiki.ros.org/dynamic_reconfigure/Tutorials
+            For an example, see http://wiki.ros.org/dynamic_reconfigure/Tutorials
 
-Have a nice day
-            """
-            print(msg)
-            sys.exit(1)
+            Have a nice day
+            """))
         self.dynconfpath = sys.argv[1]  # FIXME this is awful
         self.binary_dir = sys.argv[2]
         self.cpp_gen_dir = sys.argv[3]
@@ -317,7 +316,7 @@ Have a nice day
                 # causes imports of dynamic_reconfigure.msg to fail from at
                 # least some .cfg files. (Not sure why)
                 return
-        except:
+        except Exception:
             pass
         try:
             self.pkgname = pkgname
@@ -344,80 +343,62 @@ Have a nice day
 
     def generatewikidoc(self):
         self.mkdirabs(os.path.join(self.binary_dir, "docs"))
-        f = open(os.path.join(self.binary_dir, "docs", self.msgname + ".wikidoc"), 'w')
-        print(
-            """# Autogenerated param section. Do not hand edit.
-            param {
-            group.0 {
-            name=Dynamically Reconfigurable Parameters
-            desc=See the [[dynamic_reconfigure]] package for details on dynamically reconfigurable parameters.""", file=f)
-        i = -1
-        for param in self.group.get_parameters():
-            i = i + 1
-            range = ""
-            try:
-                enum = eval(param['edit_method'])['enum']
-                range = ", ".join(Template("$name ($value): $description").substitute(const) for const in enum)
-                range = "Possible values are: " + range
-            except:
-                if param['type'] == int_t or param['type'] == double_t:
-                    range = Template("Range: $min to $max").substitute(param)
-            print(Template(
-                  """$i.name= ~$name
-                  $i.default= $default
-                  $i.type= $type
-                  $i.desc=$description $range"""
-                  ).substitute(param, range=range, i=i), file=f)
-        print("}\n}\n# End of autogenerated section. You may edit below.", file=f)
-        f.close()
-
-    def generateusage(self):
-        self.mkdirabs("docs")
-        f = open(os.path.join(self.binary_dir, "docs", self.msgname + "-usage.dox"), 'w')
-        # print("/**", file=f)
-        print("\\subsubsection usage Usage", file=f)
-        print('\\verbatim', file=f)
-        print(Template('<node name="$nodename" pkg="$pkgname" type="$nodename">').substitute(
-            pkgname=self.pkgname, nodename=self.nodename), file=f)
-        for param in self.group.get_parameters():
-            print(Template('  <param name="$name" type="$type" value="$default" />').substitute(param), file=f)
-        print('</node>', file=f)
-        print('\\endverbatim', file=f)
-        print('', file=f)
-        # print("*/", file=f)
-        f.close()
+        with open(os.path.join(self.binary_dir, "docs", self.msgname + ".wikidoc"), 'w') as f:
+            print(textwrap.dedent("""\
+                # Autogenerated param section. Do not hand edit.
+                param {
+                group.0 {
+                name=Dynamically Reconfigurable Parameters
+                desc=See the [[dynamic_reconfigure]] package for details on dynamically reconfigurable parameters."""
+            ), file=f)
+            i = -1
+            for param in self.group.get_parameters():
+                i = i + 1
+                range = ""
+                try:
+                    enum = eval(param['edit_method'])['enum']
+                    range = ", ".join(Template("$name ($value): $description").substitute(const) for const in enum)
+                    range = "Possible values are: " + range
+                except Exception:
+                    if param['type'] == int_t or param['type'] == double_t:
+                        range = Template("Range: $min to $max").substitute(param)
+                print(Template(textwrap.dedent("""\
+                    $i.name= ~$name
+                    $i.default= $default
+                    $i.type= $type
+                    $i.desc=$description $range"""
+                )).substitute(param, range=range, i=i), file=f)
+            print("}\n}\n# End of autogenerated section. You may edit below.", file=f)
 
     def generatedoc(self):
         self.mkdirabs("docs")
         dir_path = os.path.join(self.binary_dir, "docs")
         self.mkdirabs(dir_path)
-        f = open(os.path.join(dir_path, self.msgname + ".dox"), 'w')
-        # print("/**", file=f)
-        print("\\subsubsection parameters ROS parameters", file=f)
-        print("", file=f)
-        print("Reads and maintains the following parameters on the ROS server", file=f)
-        print("", file=f)
-        for param in self.group.get_parameters():
-            print(Template("- \\b \"~$name\" : \\b [$type] $description min: $min, default: $default, max: $max").substitute(param), file=f)
-        print("", file=f)
-        # print("*/", file=f)
-        f.close()
+        with open(os.path.join(dir_path, self.msgname + ".dox"), 'w') as f:
+            # print("/**", file=f)
+            print("\\subsubsection parameters ROS parameters", file=f)
+            print("", file=f)
+            print("Reads and maintains the following parameters on the ROS server", file=f)
+            print("", file=f)
+            for param in self.group.get_parameters():
+                print(Template("- \\b \"~$name\" : \\b [$type] $description min: $min, default: $default, max: $max").substitute(param), file=f)
+            print("", file=f)
+            # print("*/", file=f)
 
     def generateusage(self):
         self.mkdirabs("docs")
-        f = open(os.path.join(self.binary_dir, "docs", self.msgname + "-usage.dox"), 'w')
-        # print("/**", file=f)
-        print("\\subsubsection usage Usage", file=f)
-        print('\\verbatim', file=f)
-        print(Template('<node name="$nodename" pkg="$pkgname" type="$nodename">').substitute(
-            pkgname=self.pkgname, nodename=self.nodename), file=f)
-        for param in self.group.get_parameters():
-            print(Template('  <param name="$name" type="$type" value="$default" />').substitute(param), file=f)
-        print('</node>', file=f)
-        print('\\endverbatim', file=f)
-        print("", file=f)
-        # print("*/", file=f)
-        f.close()
+        with open(os.path.join(self.binary_dir, "docs", self.msgname + "-usage.dox"), 'w') as f:
+            # print("/**", file=f)
+            print("\\subsubsection usage Usage", file=f)
+            print('\\verbatim', file=f)
+            print(Template('<node name="$nodename" pkg="$pkgname" type="$nodename">').substitute(
+                pkgname=self.pkgname, nodename=self.nodename), file=f)
+            for param in self.group.get_parameters():
+                print(Template('  <param name="$name" type="$type" value="$default" />').substitute(param), file=f)
+            print('</node>', file=f)
+            print('\\endverbatim', file=f)
+            print("", file=f)
+            # print("*/", file=f)
 
     def crepr(self, param, val):
         type = param["type"]
@@ -470,16 +451,15 @@ Have a nice day
         templatelines = []
         templatefilesafe = templatefile.replace('\\', '\\\\')  # line directive does backslash expansion.
         curline = 1
-        f = open(templatefile)
-        for line in f:
-            curline = curline + 1
-            templatelines.append(Template(line).safe_substitute(linenum=curline, filename=templatefilesafe))
-        f.close()
+        with open(templatefile) as f:
+            for line in f:
+                curline = curline + 1
+                templatelines.append(Template(line).safe_substitute(linenum=curline, filename=templatefilesafe))
+
         template = ''.join(templatelines)
 
         # Write the configuration manipulator.
         self.mkdirabs(self.cpp_gen_dir)
-        f = open(os.path.join(self.cpp_gen_dir, self.name + "Config.h"), 'w')
         paramdescr = []
         groups = []
         members = []
@@ -524,11 +504,11 @@ Have a nice day
         members = '\n'.join(members)
         constants = '\n'.join(constants)
         groups = '\n'.join(groups)
-        f.write(Template(template).substitute(
-            uname=self.name.upper(),
-            configname=self.name, pkgname=self.pkgname, paramdescr=paramdescr,
-            members=members, groups=groups, doline=LINEDEBUG, constants=constants))
-        f.close()
+        with open(os.path.join(self.cpp_gen_dir, self.name + "Config.h"), 'w') as f:
+            f.write(Template(template).substitute(
+                uname=self.name.upper(),
+                configname=self.name, pkgname=self.pkgname, paramdescr=paramdescr,
+                members=members, groups=groups, doline=LINEDEBUG, constants=constants))
         print("Wrote header file in " + os.path.join(self.cpp_gen_dir, self.name + "Config.h"))
 
 # def deleteoneobsolete(self, file):
@@ -602,22 +582,20 @@ Have a nice day
     def generatepy(self):
         # Read the configuration manipulator template and insert line numbers and file name into template.
         templatefile = os.path.join(self.dynconfpath, "templates", "ConfigType.py.template")
-        f = open(templatefile)
-        template = f.read()
-        f.close()
+        with open(templatefile) as f:
+            template = f.read()
 
         # Write the configuration manipulator.
         self.mkdirabs(os.path.join(self.py_gen_dir, "cfg"))
-        f = open(os.path.join(self.py_gen_dir, "cfg", self.name + "Config.py"), 'w')
-        pycfgdata = self.replace_infinity(self.group.to_dict())
-        f.write(Template(template).substitute(
-            name=self.name,
-            pkgname=self.pkgname, pycfgdata=pycfgdata))
-        for const in self.constants:
-            f.write(Template("${configname}_${name} = $v\n").substitute(
-                const, v=repr(const['value']),
-                configname=self.name))
-        f.close()
+        with open(os.path.join(self.py_gen_dir, "cfg", self.name + "Config.py"), 'w') as f:
+            pycfgdata = self.replace_infinity(self.group.to_dict())
+            f.write(Template(template).substitute(
+                name=self.name,
+                pkgname=self.pkgname, pycfgdata=pycfgdata))
+            for const in self.constants:
+                f.write(Template("${configname}_${name} = $v\n").substitute(
+                    const, v=repr(const['value']),
+                    configname=self.name))
 
-        f = open(os.path.join(self.py_gen_dir, "cfg", "__init__.py"), 'a')
-        f.close()
+        with open(os.path.join(self.py_gen_dir, "cfg", "__init__.py"), 'a'):
+            pass
